@@ -12,6 +12,7 @@ import idna  # implements IDNA 2008; Python's codec is only IDNA 2003
 ALLOW_SMTPUTF8 = True
 CHECK_DELIVERABILITY = True
 TEST_ENVIRONMENT = False
+GLOBALLY_DELIVERABLE = True
 DEFAULT_TIMEOUT = 15  # secs
 
 # Based on RFC 2822 section 3.2.4 / RFC 5322 section 3.2.3, these
@@ -251,7 +252,9 @@ def __get_length_reason(addr, utf8=False, limit=EMAIL_MAX_LENGTH):
     return reason.format(prefix, diff, suffix)
 
 
-def caching_resolver(timeout=DEFAULT_TIMEOUT, cache=None):
+def caching_resolver(*, timeout=None, cache=None):
+    if timeout is None:
+        timeout = DEFAULT_TIMEOUT
     resolver = dns.resolver.Resolver()
     resolver.cache = cache or dns.resolver.LRUCache()
     resolver.lifetime = timeout  # timeout, in seconds
@@ -260,11 +263,14 @@ def caching_resolver(timeout=DEFAULT_TIMEOUT, cache=None):
 
 def validate_email(
     email,
-    allow_smtputf8=ALLOW_SMTPUTF8,
+    # /, # not supported in Python 3.6, 3.7
+    *,
+    allow_smtputf8=None,
     allow_empty_local=False,
-    check_deliverability=CHECK_DELIVERABILITY,
-    test_environment=TEST_ENVIRONMENT,
-    timeout=DEFAULT_TIMEOUT,
+    check_deliverability=None,
+    test_environment=None,
+    globally_deliverable=GLOBALLY_DELIVERABLE,
+    timeout=None,
     dns_resolver=None
 ):
     """
@@ -273,6 +279,16 @@ def validate_email(
     but if bytes it must be ASCII-only.
     """
 
+    # Fill in default values of arguments.
+    if allow_smtputf8 is None:
+        allow_smtputf8 = ALLOW_SMTPUTF8
+    if check_deliverability is None:
+        check_deliverability = CHECK_DELIVERABILITY
+    if test_environment is None:
+        test_environment = TEST_ENVIRONMENT
+    if timeout is None:
+        timeout = DEFAULT_TIMEOUT
+
     # Allow email to be a str or bytes instance. If bytes,
     # it must be ASCII because that's how the bytes work
     # on the wire with SMTP.
@@ -300,7 +316,7 @@ def validate_email(
     ret.smtputf8 = local_part_info["smtputf8"]
 
     # Validate the email address's domain part syntax and get a normalized form.
-    domain_part_info = validate_email_domain_part(parts[1], test_environment=test_environment)
+    domain_part_info = validate_email_domain_part(parts[1], test_environment=test_environment, globally_deliverable=globally_deliverable)
     ret.domain = domain_part_info["domain"]
     ret.ascii_domain = domain_part_info["ascii_domain"]
 
@@ -356,9 +372,8 @@ def validate_email(
         deliverability_info = validate_email_deliverability(
             ret["domain"], ret["domain_i18n"], timeout, dns_resolver
         )
-        if "mx" in deliverability_info:
-            ret.mx = deliverability_info["mx"]
-            ret.mx_fallback_type = deliverability_info["mx-fallback"]
+        for key, value in deliverability_info.items():
+            setattr(ret, key, value)
 
     return ret
 
@@ -460,7 +475,7 @@ def validate_email_local_part(local, allow_smtputf8=True, allow_empty_local=Fals
         }
 
 
-def validate_email_domain_part(domain, test_environment=False):
+def validate_email_domain_part(domain, test_environment=False, globally_deliverable=True):
     # Empty?
     if len(domain) == 0:
         raise EmailSyntaxError("There must be something after the @-sign.")
@@ -538,16 +553,22 @@ def validate_email_domain_part(domain, test_environment=False):
     if not m:
         raise EmailSyntaxError("The email address contains invalid characters after the @-sign.")
 
-    # All publicly deliverable addresses have domain named with at least
-    # one period, and we'll consider the lack of a period a syntax error
-    # since that will match people's sense of what an email address looks
-    # like. We'll skip this in test environments to allow '@test' email
-    # addresses.
-    if "." not in ascii_domain and not (ascii_domain == "test" and test_environment):
-        raise EmailSyntaxError("The domain name %s is not valid. It should have a period." % domain_i18n)
+    if globally_deliverable:
+        # All publicly deliverable addresses have domain named with at least
+        # one period, and we'll consider the lack of a period a syntax error
+        # since that will match people's sense of what an email address looks
+        # like. We'll skip this in test environments to allow '@test' email
+        # addresses.
+        if "." not in ascii_domain and not (ascii_domain == "test" and test_environment):
+            raise EmailSyntaxError("The domain name %s is not valid. It should have a period." % domain_i18n)
 
-    # Check special-use and reserved domain names. Raise these as
-    # deliverability errors since they are syntactically valid.
+        # We also know that all TLDs currently end with a letter.
+        if not re.search(r"[A-Za-z]\Z", ascii_domain):
+            raise EmailSyntaxError(
+                "The domain name %s is not valid. It is not within a valid top-level domain." % domain_i18n
+            )
+
+    # Check special-use and reserved domain names.
     # Some might fail DNS-based deliverability checks, but that
     # can be turned off, so we should fail them all sooner.
     for d in SPECIAL_USE_DOMAIN_NAMES:
@@ -556,14 +577,7 @@ def validate_email_domain_part(domain, test_environment=False):
             continue
 
         if ascii_domain == d or ascii_domain.endswith("." + d):
-            raise EmailUndeliverableError("The domain name %s is a special-use or reserved name that cannot be used with email." % domain_i18n)
-
-    # We also know that all TLDs currently end with a letter, and
-    # we'll consider that a non-DNS based deliverability check.
-    if not re.search(r"[A-Za-z]\Z", ascii_domain):
-        raise EmailUndeliverableError(
-            "The domain name %s is not valid. It is not within a valid top-level domain." % domain_i18n
-        )
+            raise EmailSyntaxError("The domain name %s is a special-use or reserved name that cannot be used with email." % domain_i18n)
 
     # Return the IDNA ASCII-encoded form of the domain, which is how it
     # would be transmitted on the wire (except when used with SMTPUTF8
@@ -580,6 +594,8 @@ def validate_email_domain_part(domain, test_environment=False):
 def validate_email_deliverability(domain, domain_i18n, timeout=DEFAULT_TIMEOUT, dns_resolver=None):
     # Check that the domain resolves to an MX record. If there is no MX record,
     # try an A or AAAA record which is a deprecated fallback for deliverability.
+    # (Note that changing the DEFAULT_TIMEOUT module-level attribute
+    #  will not change the default value of this method's timeout argument.)
 
     # If no dns.resolver.Resolver was given, get dnspython's default resolver.
     # Override the default resolver's timeout. This may affect other uses of
@@ -588,6 +604,8 @@ def validate_email_deliverability(domain, domain_i18n, timeout=DEFAULT_TIMEOUT,
         dns_resolver = dns.resolver.get_default_resolver()
         dns_resolver.lifetime = timeout
 
+    deliverability_info = {}
+
     def dns_resolver_resolve_shim(domain, record):
         try:
             # dns.resolver.Resolver.resolve is new to dnspython 2.x.
@@ -611,39 +629,63 @@ def validate_email_deliverability(domain, domain_i18n, timeout=DEFAULT_TIMEOUT,
             raise dns.exception.Timeout()
 
         try:
-            # Try resolving for MX records and get them in sorted priority order
-            # as (priority, qname) pairs.
+            # Try resolving for MX records.
             response = dns_resolver_resolve_shim(domain, "MX")
-            mtas = sorted([(r.preference, str(r.exchange).rstrip('.')) for r in response])
-            mx_fallback = None
 
-            # Do not permit delivery if there is only a "null MX" record (whose value is
-            # (0, ".") but we've stripped trailing dots, so the 'exchange' is just "").
+            # For reporting, put them in priority order and remove the trailing dot in the qnames.
+            mtas = sorted([(r.preference, str(r.exchange).rstrip('.')) for r in response])
+
+            # Remove "null MX" records from the list (their value is (0, ".") but we've stripped
+            # trailing dots, so the 'exchange' is just ""). If there was only a null MX record,
+            # email is not deliverable.
             mtas = [(preference, exchange) for preference, exchange in mtas
                     if exchange != ""]
             if len(mtas) == 0:
                 raise EmailUndeliverableError("The domain name %s does not accept email." % domain_i18n)
 
+            deliverability_info["mx"] = mtas
+            deliverability_info["mx_fallback_type"] = None
+
         except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
 
-            # If there was no MX record, fall back to an A record.
+            # If there was no MX record, fall back to an A record, as SMTP servers do.
             try:
                 response = dns_resolver_resolve_shim(domain, "A")
-                mtas = [(0, str(r)) for r in response]
-                mx_fallback = "A"
+                deliverability_info["mx"] = [(0, str(r)) for r in response]
+                deliverability_info["mx_fallback_type"] = "A"
             except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
 
                 # If there was no A record, fall back to an AAAA record.
                 try:
                     response = dns_resolver_resolve_shim(domain, "AAAA")
-                    mtas = [(0, str(r)) for r in response]
-                    mx_fallback = "AAAA"
+                    deliverability_info["mx"] = [(0, str(r)) for r in response]
+                    deliverability_info["mx_fallback_type"] = "AAAA"
                 except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
 
                     # If there was no MX, A, or AAAA record, then mail to
                     # this domain is not deliverable.
                     raise EmailUndeliverableError("The domain name %s does not exist." % domain_i18n)
 
+            # Check for a SPF reject-all record ("v=spf1 -all") which indicates
+            # no emails are sent from this domain (similar to a NULL MX record
+            # but for sending rather than receiving). In combination with the
+            # absence of an MX record, this is probably a good sign that the
+            # domain is not used for email.
+            try:
+                response = dns_resolver_resolve_shim(domain, "TXT")
+                for rec in response:
+                    value = b"".join(rec.strings)
+                    if value.startswith(b"v=spf1 "):
+                        deliverability_info["spf"] = value.decode("ascii", errors='replace')
+                        if value == b"v=spf1 -all":
+                            raise EmailUndeliverableError("The domain name %s does not send email." % domain_i18n)
+            except dns.resolver.NoAnswer:
+                # No TXT records means there is no SPF policy, so we cannot take any action.
+                pass
+            except (dns.resolver.NoNameservers, dns.resolver.NXDOMAIN):
+                # Failure to resolve at this step will be ignored.
+                pass
+
     except dns.exception.Timeout:
         # A timeout could occur for various reasons, so don't treat it as a failure.
         return {
@@ -660,10 +702,7 @@ def validate_email_deliverability(domain, domain_i18n, timeout=DEFAULT_TIMEOUT,
             "There was an error while checking if the domain name in the email address is deliverable: " + str(e)
         )
 
-    return {
-        "mx": mtas,
-        "mx-fallback": mx_fallback,
-    }
+    return deliverability_info
 
 
 def main():
